#!/bin/python3
#%%
import json
import os
import numpy as np
import pandas as pd

from hyperopt import hp, STATUS_OK

from tools.std_lib.log import ColorfulLogger

from tools.data_processing.basic_data_tools import dict_to_plain

from tools.modeling.training_tools_v2 import easy_make_classification
from tools.modeling.training_tools_v2 import BinaryClassifierTrainer, BinaryClassifier

from tools.modeling.hyperopt_tools_v2 import hyperopt_fmin


log = ColorfulLogger(basename='test')

def get_xgb_space(d: dict):

    n_estimators = d.get('n_estimators', None)
    learning_rate = d.get('learning_rate', None)
    max_depth = d.get('max_depth', None)
    subsample = d.get('subsample', None)
    colsample_bytree = d.get('colsample_bytree', None)
    gamma = d.get('gamma', None)
    reg_alpha = d.get('reg_alpha', None)
    min_child_weight = d.get('min_child_weight', None)

    hyperopt_space = {}

    if n_estimators is not None:
        hyperopt_space['n_estimators'] = hp.randint(
            "n_estimators", n_estimators[0], n_estimators[1])
    
    if learning_rate is not None:
        hyperopt_space['learning_rate'] = hp.uniform(
            'learning_rate', learning_rate[0], learning_rate[1])
    
    if max_depth is not None:
        hyperopt_space['max_depth'] = hp.choice(
            "max_depth", max_depth)
    
    if subsample is not None:
        hyperopt_space['subsample'] = hp.uniform(
            "subsample", subsample[0], subsample[1])
    
    if colsample_bytree is not None:
        hyperopt_space['colsample_bytree'] = hp.uniform(
            "colsample_bytree", colsample_bytree[0], colsample_bytree[1])
    
    if gamma is not None:
        hyperopt_space['gamma'] = hp.uniform(
            "gamma", gamma[0], gamma[1])
    
    if reg_alpha is not None:
        hyperopt_space['reg_alpha'] = hp.uniform(
            "reg_alpha", reg_alpha[0], reg_alpha[1])
    
    if min_child_weight is not None:
        hyperopt_space['min_child_weight'] = hp.randint(
            "min_child_weight", min_child_weight[0], min_child_weight[1])
    
    return hyperopt_space


def xgb_loss_func(classifier_params: dict, searching = True):

    func_name = 'xgb_loss_func()'
    log.info('classifier_params =', dict_to_plain(classifier_params), func = func_name, color = 'green')

    binary_classifier = BinaryClassifier(
        X_df = X_df,
        y_df = y_df,
        classifier_name = 'XGB',
        fillna = None,
        classifier_params = classifier_params,
        prefix = 'prefix',
        cache_dir = './cache/clf/',
        train_cache_dir= './cache/train/'
    )
    binary_classifier.run()
    evaluation_dict = binary_classifier.evaluation_dict
    auc_ks_by_data_split = evaluation_dict['auc_ks_by_data_split']

    train_ks = auc_ks_by_data_split['ks']['train']
    validate_ks = auc_ks_by_data_split['ks']['validate']
    test_ks = auc_ks_by_data_split['ks']['test']

    loss = 1 - train_ks
    # loss = (1 - (validate_ks+test_ks)/2)
    # loss = (1 - (validate_ks+test_ks)/2) + abs(train_ks - validate_ks)

    loss_dict = {
            'loss': loss, 
            'status': STATUS_OK,
            }
    log.info(f'new_loss = {loss}', func = func_name)

    return loss_dict


#%%

X_df, y_df = easy_make_classification(
    n_sample=140000,
    # weights=[.6, .4],
    weights=[.98, .02],
    n_features=100,
    class_sep= 1,
    test_size=2/6,
    flip_y=.2
    )

d = {
    "n_estimators":  [50, 201],
    'learning_rate': [.001, .20],
    "max_depth":  [2, 3],

    "subsample":  [.5, 1],
    "colsample_bytree":  [.5, 1],
    "gamma":  [.1, 10],
    "reg_alpha":  [.1, 10],
    "min_child_weight":  [10, 1001],
    }

space = get_xgb_space(d)
max_evals = 1000
iteration_stop_count = 50

best_params, best_loss, trials = hyperopt_fmin(
    hyperopt_loss_func=xgb_loss_func,
    space=space,
    max_evals = max_evals,
    iteration_stop_count = iteration_stop_count
)

#%%
